@@ -1,18 +1,23 @@
+local base = require("packages.base")
+
+local package = pl.class(base)
+package._name = "tableofcontents"
+
 if not SILE.scratch._tableofcontents then
   SILE.scratch._tableofcontents = {}
 end
 
-local function _moveTocNodes (class)
+function package:moveTocNodes ()
   local node = SILE.scratch.info.thispage.toc
   if node then
     for i = 1, #node do
-      node[i].pageno = class:formatCounter(SILE.scratch.counters.folio)
+      node[i].pageno = self.packages.counters:formatCounter(SILE.scratch.counters.folio)
       table.insert(SILE.scratch.tableofcontents, node[i])
     end
   end
 end
 
-local function _writeToc (_)
+function package.writeToc (_)
   local tocdata = pl.pretty.write(SILE.scratch.tableofcontents)
   local tocfile, err = io.open(SILE.masterFilename .. '.toc', "w")
   if not tocfile then return SU.error(err) end
@@ -24,6 +29,21 @@ local function _writeToc (_)
   end
 end
 
+function package.readToc (_)
+  if SILE.scratch._tableofcontents and #SILE.scratch._tableofcontents > 0 then
+    -- already loaded
+    return SILE.scratch._tableofcontents
+  end
+  local tocfile, _ = io.open(SILE.masterFilename .. '.toc')
+  if not tocfile then
+    return false -- No TOC yet
+  end
+  local doc = tocfile:read("*all")
+  local toc = assert(load(doc))()
+  SILE.scratch._tableofcontents = toc
+  return SILE.scratch._tableofcontents
+end
+
 local function _linkWrapper (dest, func)
   if dest and SILE.Commands["pdf:link"] then
     return function()
@@ -64,32 +84,29 @@ if not SILE.scratch.pdf_destination_counter then
   SILE.scratch.pdf_destination_counter = 1
 end
 
-local function init (class, _)
-
+function package:_init ()
+  base._init(self)
   if not SILE.scratch.tableofcontents then
     SILE.scratch.tableofcontents = {}
   end
-
-  class:loadPackage("infonode")
-  class:loadPackage("leaders")
-
-  class:registerHook("endpage", _moveTocNodes)
-  class:registerHook("finish", _writeToc)
-
+  self.class:loadPackage("infonode")
+  self.class:loadPackage("leaders")
+  self.class:registerHook("endpage", self.moveTocNodes)
+  self.class:registerHook("finish", self.writeToc)
+  self:deprecatedExport("writeToc", self.writeToc)
+  self:deprecatedExport("moveTocNodes", self.moveTocNodes)
 end
 
-local function registerCommands (_)
+function package:registerCommands ()
 
-  SILE.registerCommand("tableofcontents", function (options, _)
+  self:registerCommand("tableofcontents", function (options, _)
     local depth = SU.cast("integer", options.depth or 3)
     local linking = SU.boolean(options.linking, true)
-    local tocfile,_ = io.open(SILE.masterFilename .. '.toc')
-    if not tocfile then
+    local toc = self:readToc()
+    if toc == false then
       SILE.call("tableofcontents:notocmessage")
       return
     end
-    local doc = tocfile:read("*all")
-    local toc = assert(load(doc))()
     SILE.call("tableofcontents:header")
     for i = 1, #toc do
       local item = toc[i]
@@ -103,10 +120,9 @@ local function registerCommands (_)
       end
     end
     SILE.call("tableofcontents:footer")
-    SILE.scratch._tableofcontents = toc
   end)
 
-  SILE.registerCommand("tableofcontents:item", function (options, content)
+  self:registerCommand("tableofcontents:item", function (options, content)
     SILE.settings:temporarily(function ()
       SILE.settings:set("typesetter.parfillskip", SILE.nodefactory.glue())
       SILE.call("tableofcontents:level" .. options.level .. "item", {
@@ -127,7 +143,7 @@ local function registerCommands (_)
     end)
   end)
 
-  SILE.registerCommand("tocentry", function (options, content)
+  self:registerCommand("tocentry", function (options, content)
     local dest
     if SILE.Commands["pdf:destination"] then
       dest = "dest" .. tostring(SILE.scratch.pdf_destination_counter)
@@ -150,81 +166,59 @@ local function registerCommands (_)
     })
   end)
 
-  SILE.registerCommand("tableofcontents:title", function (_, _)
-    SU.deprecated("\\tableofcontents:title", "\\fluent{toc-title}", "0.13.0", "0.14.0")
+  self:registerCommand("tableofcontents:title", function (_, _)
+    SU.deprecated("\\tableofcontents:title", "\\fluent{tableofcontents-title}", "0.13.0", "0.14.0")
   end, "Deprecated")
 
-  SILE.registerCommand("tableofcontents:notocmessage", function (_, _)
+  self:registerCommand("tableofcontents:notocmessage", function (_, _)
     SILE.call("tableofcontents:headerfont", {}, function ()
-      SILE.call("fluent", {}, { "toc-not-generated" })
+      SILE.call("fluent", {}, { "tableofcontents-not-generated" })
     end)
   end)
 
-  SILE.registerCommand("tableofcontents:headerfont", function (_, content)
+  self:registerCommand("tableofcontents:headerfont", function (_, content)
     SILE.call("font", { size = 24, weight = 800 }, content)
   end)
 
-  SILE.registerCommand("tableofcontents:header", function (_, _)
+  self:registerCommand("tableofcontents:header", function (_, _)
     SILE.call("par")
     SILE.call("noindent")
     SILE.call("tableofcontents:headerfont", {}, function ()
-      SILE.call("fluent", {}, { "toc-title" })
+      SILE.call("fluent", {}, { "tableofcontents-title" })
     end)
     SILE.call("medskip")
   end)
 
-  SILE.registerCommand("tableofcontents:footer", function (_, _) end)
+  self:registerCommand("tableofcontents:footer", function (_, _) end)
 
-  SILE.registerCommand("tableofcontents:level1item", function (_, content)
+  self:registerCommand("tableofcontents:level1item", function (_, content)
     SILE.call("bigskip")
     SILE.call("noindent")
     SILE.call("font", { size = 14, weight = 800 }, content)
     SILE.call("medskip")
   end)
 
-  SILE.registerCommand("tableofcontents:level2item", function (_, content)
+  self:registerCommand("tableofcontents:level2item", function (_, content)
     SILE.call("noindent")
     SILE.call("font", { size = 12 }, content)
     SILE.call("medskip")
   end)
 
-  SILE.registerCommand("tableofcontents:level3item", function (_, content)
+  self:registerCommand("tableofcontents:level3item", function (_, content)
     SILE.call("indent")
     SILE.call("font", { size = 10 }, content)
     SILE.call("smallskip")
   end)
 
-  SILE.registerCommand("tableofcontents:level1number", function (_, _) end)
+  self:registerCommand("tableofcontents:level1number", function (_, _) end)
 
-  SILE.registerCommand("tableofcontents:level2number", function (_, _) end)
+  self:registerCommand("tableofcontents:level2number", function (_, _) end)
 
-  SILE.registerCommand("tableofcontents:level3number", function (_, _) end)
+  self:registerCommand("tableofcontents:level3number", function (_, _) end)
 
 end
 
-local _deprecate  = [[
-  Directly calling tableofcontents handling functions is no longer necessary.
-  All the SILE core classes and anything inheriting from them will take care of
-  this automatically using hooks. Custom classes that override the
-  class:endPage() and class:finish() functions may need to handle this in other
-  ways. By calling these hooks directly you are likely causing them to run
-  twice and duplicate entries.
-]]
-
-return {
-  init = init,
-  registerCommands = registerCommands,
-  exports = {
-    writeToc = function (_)
-      SU.deprecated("class:writeToc", nil, "0.13.0", "0.14.0", _deprecate)
-      return _writeToc()
-    end,
-    moveTocNodes = function (class)
-      SU.deprecated("class:moveTocNodes", nil, "0.13.0", "0.14.0", _deprecate)
-      return _moveTocNodes(class)
-    end
-  },
-  documentation = [[
+package.documentation = [[
 \begin{document}
 The \autodoc:package{tableofcontents} package provides tools for class authors to
 create tables of contents. When you are writing sectioning commands such
@@ -262,4 +256,5 @@ following commands:
 
 \end{document}
 ]]
-}
+
+return package
